var XWiki = (function(XWiki) {
  document.observe('xwiki:dom:loaded', function() {
    DOCUMENT_URL = "$escapetool.javascript($request.getParameter('documentUrl'))?xpage=shareinline";
    Ajax.Request.prototype.setRequestHeaders = Ajax.Request.prototype.setRequestHeaders.wrap(function(setHeaders) {
      setHeaders();
      if (this.options.xhrFields) Object.extend(this.transport, this.options.xhrFields);
    });
    /**
     * Extends the SuggestPicker to customize the way the selected item icons are displayed (instead of using avatars by
     * default since they would take too much visual space).
     */
    var TargetSelectionManager = Class.create(XWiki.widgets.SuggestPicker, {
      // @Override
      initialize: function($super, element, suggest, options) {
        $super(element, suggest, options);
        // Needed to display properly the icon next to the recipient's type
        this.list.addClassName('targets-container');
      },

      // @Override
      matchesSelectedValue: function(value, suggestion) {
        // The given value can be a relative user reference. We need to parse it and match only the user alias.
        return XWiki.Model.resolve(value, XWiki.EntityType.DOCUMENT).name == suggestion.id;
      },

      displayItemInternal: function(suggestion, itemClass) {
        var targetInfo = new Element('span').update(suggestion.info).insert(this.createDeleteTool());
        var targetClass = 'target-item ' + itemClass;
        return new Element('li', {'class': targetClass}).insert(targetInfo).insert(this.createItemInput(suggestion));
      }

    });

    var UserSelectionManager = Class.create(TargetSelectionManager, {
      // @Override
      displayItem: function(suggestion) {
        return this.displayItemInternal(suggestion, 'target-user');
      }
    });

    var GroupSelectionManager = Class.create(TargetSelectionManager, {
      // @Override
      displayItem: function(suggestion) {
        return this.displayItemInternal(suggestion, 'target-group');
      }
    });

    var EmailSelectionManager = Class.create(TargetSelectionManager, {
      // @Override
      displayItem: function(suggestion) {
        return this.displayItemInternal(suggestion, 'target-email');
      }
    });

    /**
     * Extends the UserPicker to change the selection manager for users.
     */
    var CustomUserPicker = Class.create(XWiki.widgets.UserPicker, {
      // @Override
      _createSelectionManager: function(options) {
        return new UserSelectionManager(this.fld, this, options);
      },
    });

    /**
     * Extends the UserPicker to change the selection manager for groups.
     */
    var CustomGroupPicker = Class.create(XWiki.widgets.UserPicker, {
      // @Override
      _createSelectionManager: function(options) {
        return new GroupSelectionManager(this.fld, this, options);
      },
    });

    XWiki.SharePage = Class.create({
      initialize : function () {
        this.inputUsers = $('shareTargetUsers');
        this.inputGroups = $('shareTargetGroups');
        this.inputEmails = $('shareTargetEmails');
        this.shareErrorDiv = $('shareErrorField');

        if (!this.inputUsers || !this.inputGroups) {
          return;
        }

        new CustomUserPicker(this.inputUsers, {
          script: XWiki.currentDocument.getURL('get', 'xpage=uorgsuggest&uorg=user&'),
          shownoresults : true
        });

        new CustomGroupPicker(this.inputGroups, {
          script: XWiki.currentDocument.getURL('get', 'xpage=uorgsuggest&uorg=group&'),
          shownoresults : true
        });

        this.inputUsers.focus();

        document.observe('xwiki:multisuggestpicker:selectionchanged', this._onSelectionChanged.bindAsEventListener(this));

        this.form = this.inputUsers.up('form');
        if (this.form) {
          this.form.observe('submit', this._onSubmit.bindAsEventListener(this));
          dialog && this.form.down('.secondary').observe('click', function(event) {
            event.stop();
            dialog.closeDialog();
          }.bindAsEventListener(this));
        }
      },

      _onSubmit : function(event) {
        event.stop();
        if ($$('input[id^="targetUsers"]').size() == 0 && $$('input[id^="targetGroups"]').size() == 0 && this.inputEmails.value == '' ) {
          this.inputUsers.addClassName('xErrorField');
          this.inputGroups.addClassName('xErrorField');
          this.inputEmails.addClassName('xErrorField');
          // Remove any existing error message before adding the new message
          var errorMessage = this.shareErrorDiv.next('.xErrorMsg');
          errorMessage && errorMessage.remove();
          this.shareErrorDiv.insert({'after' : '<div class="xErrorMsg">Пожалуйста, введите получателя</div>'});
          return;
        }
        var params = this.form.serialize();
        this.form.up().update(loading);
        new Ajax.Updater(loading.up(), DOCUMENT_URL, {
          xhrFields: {
            withCredentials: true
          },
          parameters: params,
          onComplete : function() {
            dialog && dialog.dialog.down('.share-backlink').observe('click', dialog.closeDialog.bind(dialog));
          }
        });
      },

      _onSelectionChanged : function(event) {
        if (event.memo.trigger == this.inputUsers || event.memo.trigger == this.inputGroups || event.memo.trigger == this.inputEmails ) {
          this.inputUsers.removeClassName('xErrorField');
          this.inputGroups.removeClassName('xErrorField');
          this.inputEmails.removeClassName('xErrorField');
          // Remove any error message since a target has been added
          var errorMessage = this.shareErrorDiv.next('.xErrorMsg');
          errorMessage && errorMessage.remove();
        }
      }
    });
  });

  var loading = new Element('div', {'class' : 'imgcenter'}).update("<img src=\"../../../resources/icons/xwiki/ajax-loader-large.gif\"/>");
  var dialog;
  document.observe('xwiki:dom:loaded', function() {
    $('shareTargetUsers') && new XWiki.SharePage();
    if ($('tmActionShareButton')) {
      $('tmActionShareButton').observe('click', function(event) {
        event.stop();
        dialog = new XWiki.widgets.ModalPopup(
          loading, {}, {
            'verticalPosition' : 'top',
            'title' : "Поделиться страницей",
            'removeOnClose' : true
          }
        );
        dialog.showDialog();
        loading.up().up().addClassName('share-dialog');
        new Ajax.Updater(loading.up(), DOCUMENT_URL, {
          xhrFields: {
            withCredentials: true
          },
          onComplete : function() { 
            new XWiki.SharePage(); 
          }
        });
      });
    }
  });
  return XWiki;
}(XWiki || {}));
